"use client"

import CustomCard from "@/components/card/Card";
import { Box, Button, Flex, FormControl, FormLabel, Text, useColorModeValue, useToast } from "@chakra-ui/react";
import Form from "@/components/form/Form";
import { InputWrapper } from "@/components/fields/Fields";
import prepareValidationSchema, { FormObject } from "@/utils/validation-schema";
import { useRouter } from "next/navigation";
import { useEffect, useMemo, useRef, useState } from "react";
import { JackpotSettingInterface } from "../types/setting";
import { saveDivisionSetting, saveJackpotSetting } from "@/app/lib/setting/action";
import { useGeneral } from "@/contexts/GeneralContext";


interface formValues {
  coins: string;
  jackpot_coins_share: string;
  requiredStreak: string;
}


export default function JackpotSetting({ jackpotSetting, settingId }: { settingId:string; jackpotSetting: JackpotSettingInterface }) {
  const { isAdmin } = useGeneral();
  // Chakra imports
  const toast = useToast();
  const router = useRouter();
  // style
  const textColor = useColorModeValue("secondaryGray.900", "white");
  const brandStars = useColorModeValue("brand.500", "brand.400");
  // logic values
  const formFields: FormObject[] = [
    {
      name: "coins",
      label: "Coins",
      type: "number",
      defaultValue: jackpotSetting.coins ?? 0,
      min: 0,
      max: 10,
      required: true,
    },
    {
      name: "jackpot_coins_share",
      label: "Jackpot Share %",
      type: "number",
      defaultValue: jackpotSetting.jackpot_coins_share ?? 0,
      min: 0,
      max: 10,
      required: true,
    },
    {
      name: "requiredStreak",
      label: "Required Streak",
      defaultValue: jackpotSetting.requiredStreak ?? 0,
      type: "number",
      min: 2,
      max: 10,
      required: true,
    },
  ];
  const formRef = useRef(null);
  const [dynamicFormField, setDynamicFormField] = useState(formFields);
  const formDetails = useMemo(() => prepareValidationSchema(dynamicFormField), [dynamicFormField]);
  const [jackpotForm, setJackpotForm] = useState({});
  const { initialFormData, validationSchema } = formDetails;
  const [formValidationSchema, setFormValidationSchema] = useState({});

  // useEffect(() => {
  //   setDynamicFormField((prev) => {
  //     const formVals = Object.values(jackpotSetting);
  //     prev.forEach((val, index) => {
  //       val.defaultValue = formVals[index];
  //     });
  //     return prev;
  //   });
  // }, [jackpotSetting]);

  useEffect(() => {
    setJackpotForm(initialFormData);
    setFormValidationSchema(validationSchema);
  }, [initialFormData, validationSchema, dynamicFormField]);

  const handleSubmit = async (values: formValues, data: object) => {
    const payload = {
      coins: parseInt(values.coins),
      requiredStreak: parseInt(values.requiredStreak),
    };
    const payload2 = {
      jackpot_coins_share: parseInt(values.jackpot_coins_share),
    };

    const [res, res2] = await Promise.all([saveJackpotSetting(payload), saveDivisionSetting(settingId, payload2)]);
    
    if ((res.status == 200 || res.status ) || (res2.status == 200 || res2.status )) {
      toast({
        title: "200",
        description: res?.message ?? res2.message,
        status: "success",
        position: "top-right",
        duration: 2000,
        isClosable: true,
      });
      router.refresh();
    } else {
      toast({
        title: res.status,
        description: res.error,
        status: "error",
        position: "top-right",
        duration: 9000,
        isClosable: true,
      });
    }
  };

  return (
    <CustomCard w="100%">
      <Flex align="center" justifyContent="space-between" w="100%" px="15px" py="10px">
        <Text color={textColor} fontSize="xl" fontWeight="700" lineHeight="100%">
          Jackpot Setting
        </Text>
      </Flex>
      <Flex align="center" w="100%">
        <Form
          ref={formRef}
          validationSchema={formValidationSchema}
          initialValues={jackpotForm}
          submitForm={(formValues, formData) => {
            handleSubmit(formValues, formData);
          }}>
          <FormControl w="full">
            <Flex justifyContent="space-between">
              <Flex flexDirection="column">
                <FormLabel display="flex" ms="4px" fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                  Coins<Text color={brandStars}>*</Text>
                </FormLabel>
                <InputWrapper disabled={!isAdmin} form={true} fieldName="coins" defaultValue={jackpotSetting.coins} id="coins" placeholder="0" variant="auth" w="full" mb="2px" fontSize="sm" ms={{ base: "0px", md: "0px" }} type="number" fontWeight="500" />
              </Flex>
              <Flex flexDirection="column">
                <FormLabel display="flex" ms="4px" fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                  Jackpot Share %<Text color={brandStars}>*</Text>
                </FormLabel>
                <InputWrapper disabled={!isAdmin} form={true} fieldName="jackpot_coins_share" defaultValue={jackpotSetting.jackpot_coins_share} id="jackpot_coins_share" placeholder="0" variant="auth" w="full" mb="2px" fontSize="sm" ms={{ base: "0px", md: "0px" }} type="number" fontWeight="500" />
              </Flex>
              <Flex flexDirection="column">
                <FormLabel display="flex" ms="4px" fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                  Required Streak<Text color={brandStars}>*</Text>
                </FormLabel>
                <InputWrapper disabled={!isAdmin} form={true} fieldName="requiredStreak" defaultValue={jackpotSetting.requiredStreak} id="requiredStreak" placeholder="0" variant="auth" w="full" mb="2px" fontSize="sm" ms={{ base: "0px", md: "0px" }} type="number" fontWeight="500" />
              </Flex>
            </Flex>
            { isAdmin && <Button fontSize="sm" type="submit" variant="brand" fontWeight="500" h="50" mb="28px">
              Save
            </Button>}
          </FormControl>
        </Form>
      </Flex>
    </CustomCard>
  );
}
